#ifndef GAME_STATE_H
#define GAME_STATE_H

#include "AppState.h"
#include <OgreSubEntity.h>
#include <OgreMaterialManager.h>
#include "BtOgreExtras.h"
#include "BtOgreGP.h"
#include "BtOgrePG.h"
#include <vector>

enum QueryFlags
{
	OGRE_HEAD_MASK = 1<<0,
	CUBE_MASK = 1<<1
};

class VehicleController;

using namespace Ogre;

enum ZOMBIE_ANIMATION_TYPE
{
	ZOMBIE_ANIMATION_TYPE_SPAWN,
	ZOMBIE_ANIMATION_TYPE_WALK,
	ZOMBIE_ANIMATION_TYPE_ATTACK,
	ZOMBIE_ANIMATION_TYPE_DIE,
	ZOMBIE_ANIMATION_TYPE_BANG
};

struct ZombieAnimationState
{
	ZOMBIE_ANIMATION_TYPE type;
	long double startedAt;
	long double animationDuration;
	int numFrames;
	int startFrame;
	int endFrame;
	int currentFrame;
	Material* material;
	bool repeat;
	String name;
	int numFramesX;
	int numFramesY;
};
typedef std::vector<ZombieAnimationState>::iterator ZombieAnimationStateIterator;

class GameState : public AppState, Ogre::FrameListener
{
public:
	GameState();
	~GameState();
	DECLARE_APPSTATE_CLASS(GameState)

	void enter();
	void createScene();
	void exit();
	bool pause();
	void resume();

	void moveCamera();
	void getInput();
	void buildGUI();

	bool keyPressed(const OIS::KeyEvent &keyEventRef);
	bool keyReleased(const OIS::KeyEvent &keyEventRef);

	bool mouseMoved(const OIS::MouseEvent &arg);
	bool mousePressed(const OIS::MouseEvent &arg, OIS::MouseButtonID id);
	bool mouseReleased(const OIS::MouseEvent &arg, OIS::MouseButtonID id);

	void onLeftPressed(const OIS::MouseEvent &evt);

	void update(double timeSinceLastFrame);
	
	bool frameStarted(const FrameEvent &evt);

	
	SceneNode *createZombie(float sizeX, float sizeY, float roration, Vector3 pos, String prefix ="z");
	
	void buttonHit(OgreBites::Button *button);
	
	virtual void sliderMoved(OgreBites::Slider *slider);
private:
	Ogre::SceneNode* m_pOgreHeadNode;
	Ogre::Entity* m_pOgreHeadEntity;
	Ogre::MaterialPtr m_pOgreHeadMat;
	Ogre::MaterialPtr m_pOgreHeadMatHigh;

	Ogre::RaySceneQuery *mRaySceneQuery;// The ray scene query pointer


	OgreBites::ParamsPanel* m_pDetailsPanel;
	bool m_bQuit;

	Ogre::Vector3 m_TranslateVector;
	Ogre::Real m_MoveSpeed;
	Ogre::Degree m_RotateSpeed;
	float m_MoveScale;
	float m_MoveUpScale;
	Ogre::Degree m_RotScale;

	Ogre::RaySceneQuery* m_pRSQ;
	Ogre::SceneNode* m_pCurrentObject;
	Ogre::Entity* m_pCurrentEntity;
	bool m_bLMouseDown, m_bRMouseDown;
	bool m_bSettingsMode;
	
	/* Bullet stuff */
	btDynamicsWorld *phyWorld;
	BtOgre::DebugDrawer *dbgdraw;
	
	btAxisSweep3 *mBroadphase;
	btDefaultCollisionConfiguration *mCollisionConfig;
	btCollisionDispatcher *mDispatcher;
	btSequentialImpulseConstraintSolver *mSolver;
	
	Ogre::Entity *mGroundEntity;
	btRigidBody *mGroundBody;
	btBvhTriangleMeshShape *mGroundShape;
	
	
	btBoxShape *mSphereShape;
	
	std::vector<btRigidBody*> mSphereBodies;
	std::vector<Ogre::Entity*> mSphereEntities;
	std::vector<Ogre::SceneNode*> mSphereNodes;
	
	
	Ogre::Entity *mPlaneEntity;
	btRigidBody *mPlaneBody;
	btBvhTriangleMeshShape *mPlaneShape;
	
	VehicleController* mVehicleController;
	
	
	// Vehicle gui sliders
	OgreBites::Slider *wheelRadiusSlider;
	
	OgreBites::Slider *wheelFrictionSlider;

	OgreBites::Slider *suspensionStiffnessSlider;
	
	OgreBites::Slider* suspensionDampingSlider;
	OgreBites::Slider* suspensionCompressionSlider;
	OgreBites::Slider* rollInfluenceSlider;
	
	OgreBites::Slider *createSlider(const DisplayString &caption, float value, float maxValue = 200);
	RaySceneQuery* m_RayQuery;

	int m_zombies;
	void playZombieAnimations();

	std::vector<ZombieAnimationState> mPendingAnimations;
	ZombieAnimationState *pushZombieAnimationState(ZOMBIE_ANIMATION_TYPE type, String name, bool repeat = false, bool replace = false);
	long long getCurrentTime();


	int m_worldPlanes;
	int m_worldEnvItems;
	bool m_worldEnvSpawnLeft;

	SceneNode *createGroundPlane(float sizeX, float sizeY, double posX);
	SceneNode *createRandomEnvItem();
	void deleteElementsBehind();


	std::vector<SceneNode*> autoNodes;
	float m_lastShoot;
	void controlAnimations();
};

#endif

