#ifndef VEHICLECONTROLLER_H
#define VEHICLECONTROLLER_H


#include <vector>
#include <OgreFrameListener.h>
#include <bullet/LinearMath/btScalar.h>

class btRaycastVehicle;
class btRigidBody;
class btTransform;
class btCollisionShape;
class btDynamicsWorld;
class btDefaultVehicleRaycaster;
class btBoxShape;

using namespace std;
using namespace Ogre;

class VehicleController : public Ogre::FrameListener
{
public:
	VehicleController(btDynamicsWorld *phyWorld, SceneManager *sceneMgr);
	~VehicleController();
	
	void setUp();	
	void clientResetScene();
	
	bool frameStarted(const FrameEvent &evt);

	float wheelRadius;
	float wheelFriction;
	float suspensionStiffness;
	float suspensionDamping;
	float suspensionCompression;
	float rollInfluence;
	
	SceneNode* mCarBody;
	
	void updateWheelsParams();

private:
	btRigidBody* localCreateRigidBody(SceneNode *node, float mass, btCollisionShape* shape);
	
private:
	
	btDynamicsWorld *mPhyWorld;
	SceneManager* mSceneMgr;
	
	// Car members
	btDefaultVehicleRaycaster *m_vehicleRayCaster;
	btRaycastVehicle* m_vehicle;
	btRigidBody *m_carChassis;
	btBoxShape *chassisShape;


	SceneNode* tireNode;
	std::vector<SceneNode*> tires;
	
	float	gEngineForce;
	float	gBreakingForce;
	float	gVehicleSteering;
	
	btScalar mEngineForceForward;
	btScalar mEngineForceBack;
	btScalar mBrakeForce;
	
};

#endif // VEHICLECONTROLLER_H
