#include "GameState.h"
#include "BtOgreExtras.h"
#include "VehicleController.h"

#include <ctime>

using namespace Ogre;

GameState::GameState()
{
	m_MoveSpeed                 = 0.7f;
	m_RotateSpeed               = 0.3f;

	m_bLMouseDown       = false;
	m_bRMouseDown       = false;
	m_bQuit             = false;
	m_bSettingsMode     = false;

	m_pDetailsPanel             = 0;


//	mBroadphase = new btAxisSweep3(btVector3(-10000,-10000,-10000), btVector3(10000,10000,10000), 1024);
//	mCollisionConfig = new btDefaultCollisionConfiguration();
//	mDispatcher = new btCollisionDispatcher(mCollisionConfig);
//	mSolver = new btSequentialImpulseConstraintSolver();
//	phyWorld = new btDiscreteDynamicsWorld(mDispatcher, mBroadphase, mSolver, mCollisionConfig);
//	phyWorld->setGravity(btVector3(0,-9.8,0));

	m_zombies = 0;
}

GameState::~GameState()
{
//	std::vector<btRigidBody*>::const_iterator it = mSphereBodies.begin();

//	for(; it != mSphereBodies.end(); ++it) {
//		phyWorld->removeRigidBody(*it);
//		delete (*it)->getMotionState();
//		delete *it;
//	}

//	delete mVehicleController;
	
//	delete mSphereShape;

//	phyWorld->removeRigidBody(mPlaneBody);
//	delete mPlaneBody->getMotionState();
//	delete mPlaneBody;
//	delete mPlaneShape->getMeshInterface();
//	delete mPlaneShape;

//	delete dbgdraw;
//	delete phyWorld;

//	delete mSolver;
//	delete mDispatcher;
//	delete mCollisionConfig;
//	delete mBroadphase;
}

void GameState::enter()
{
	AdvancedOgreFramework::getSingletonPtr()->m_pLog->logMessage("Entering GameState...");

	m_pSceneMgr = AdvancedOgreFramework::getSingletonPtr()->m_pRoot->createSceneManager(ST_GENERIC, "GameSceneMgr");
	m_RayQuery = m_pSceneMgr->createRayQuery(Ray());

	m_pSceneMgr->addRenderQueueListener(AdvancedOgreFramework::getSingletonPtr()->mOverlaySystem);

	mRaySceneQuery = m_pSceneMgr->createRayQuery(Ogre::Ray());

	m_pRSQ = m_pSceneMgr->createRayQuery(Ray());

	m_pCamera = m_pSceneMgr->createCamera("GameCamera");
	m_pCamera->setPosition(Vector3(0, 20, 20));
	m_pCamera->lookAt(Vector3(0, 20, 0));
	m_pCamera->setNearClipDistance(0.1f);
	m_pCamera->setAspectRatio(Real(AdvancedOgreFramework::getSingletonPtr()->m_pViewport->getActualWidth()) /
							  Real(AdvancedOgreFramework::getSingletonPtr()->m_pViewport->getActualHeight()));


	AdvancedOgreFramework::getSingletonPtr()->m_pViewport->setCamera(m_pCamera);
	AdvancedOgreFramework::getSingletonPtr()->m_pRoot->addFrameListener(this);

	m_pCurrentObject = 0;

	buildGUI();
	createScene();
	m_lastShoot = getCurrentTime();

//	m_worldEnvItems = 0;
//	m_zombies = 0;
//	m_worldPlanes = 0;
}

OgreBites::Slider *GameState::createSlider(const DisplayString &caption, float value, float maxValue)
{
	OgreBites::SdkTrayManager* trayMgr = AdvancedOgreFramework::getSingletonPtr()->m_pTrayMgr;
	OgreBites::Slider *slider = trayMgr->createThickSlider(OgreBites::TL_TOPRIGHT, caption.asUTF8(), caption, 200, 40, 0, maxValue, 1000);
	slider->setValue(value, false);
	return slider;
}

bool GameState::pause()
{
	AdvancedOgreFramework::getSingletonPtr()->m_pLog->logMessage("Pausing GameState...");
	AdvancedOgreFramework::getSingletonPtr()->m_pRoot->removeFrameListener(this);
//	AdvancedOgreFramework::getSingletonPtr()->m_pRoot->removeFrameListener(mVehicleController);
	return true;
}

void GameState::resume()
{
	AdvancedOgreFramework::getSingletonPtr()->m_pLog->logMessage("Resuming GameState...");

	buildGUI();

	AdvancedOgreFramework::getSingletonPtr()->m_pViewport->setCamera(m_pCamera);
	AdvancedOgreFramework::getSingletonPtr()->m_pRoot->addFrameListener(this);
//	AdvancedOgreFramework::getSingletonPtr()->m_pRoot->addFrameListener(mVehicleController);
	m_bQuit = false;
}

void GameState::exit()
{
	m_pSceneMgr->destroyQuery(m_RayQuery);

	AdvancedOgreFramework::getSingletonPtr()->m_pLog->logMessage("Leaving GameState...");
	m_pSceneMgr->removeRenderQueueListener(AdvancedOgreFramework::getSingletonPtr()->mOverlaySystem);

	m_pSceneMgr->destroyCamera(m_pCamera);
	m_pSceneMgr->destroyQuery(m_pRSQ);
	if(m_pSceneMgr) {
		AdvancedOgreFramework::getSingletonPtr()->m_pRoot->destroySceneManager(m_pSceneMgr);
	}
	//	m_pSceneMgr->destroyAllEntities();
}

void GameState::createScene()
{
	// Light
	ColourValue fadeColour(0.0, 0.0, 0.01);
	m_pCamera->getViewport()->setBackgroundColour(fadeColour);
	m_pSceneMgr->setAmbientLight(Ogre::ColourValue(0.7, 0.7, 0.7));
	m_pSceneMgr->createLight("light")->setPosition(100, 300, 0);



	// Create background rectangle covering the whole screen
	Rectangle2D* rect = new Rectangle2D(true);
	rect->setCorners(-1.0, 1.0, 1.0, -1.0);
	rect->setMaterial("Background");
	// Render the background before everything else
	rect->setRenderQueueGroup(RENDER_QUEUE_BACKGROUND);
	// Set the bounding box to something big
	rect->setBoundingBox(AxisAlignedBox(-100000.0*Vector3::UNIT_SCALE, 100000.0*Vector3::UNIT_SCALE));

	// Attach background to the scene
	SceneNode* node = m_pSceneMgr->getRootSceneNode()->createChildSceneNode("Background");
	node->attachObject(rect);




	// Make copy of the material for other ground
	Material* ground2Mat;
	// Ground1 defined in file
	// Ground1 Flipped
	ground2Mat = MaterialManager::getSingletonPtr()->getByName("GroundPlane")->clone("GroundPlane1").get();
	ground2Mat->getTechnique(0)->getPass(0)->getFragmentProgramParameters()->setNamedConstant("flipped", -1);

	// Ground2
	ground2Mat = MaterialManager::getSingletonPtr()->getByName("GroundPlane")->clone("GroundPlane2").get();
	ground2Mat->getTechnique(0)->getPass(0)->getTextureUnitState(0)->setTextureName("Ground_02.tga");

	// Ground2 Flipped
	ground2Mat = MaterialManager::getSingletonPtr()->getByName("GroundPlane2")->clone("GroundPlane3").get();
	ground2Mat->getTechnique(0)->getPass(0)->getFragmentProgramParameters()->setNamedConstant("flipped", -1);

	// Tree1
	ground2Mat = MaterialManager::getSingletonPtr()->getByName("GroundPlane")->clone("Tree1").get();
	ground2Mat->getTechnique(0)->getPass(0)->getTextureUnitState(0)->setTextureName("tree1.tga");

	// Tree1 Flipped
	ground2Mat = MaterialManager::getSingletonPtr()->getByName("Tree1")->clone("Tree1Flipped").get();
	ground2Mat->getTechnique(0)->getPass(0)->getFragmentProgramParameters()->setNamedConstant("flipped", -1);

	// Grave1
	ground2Mat = MaterialManager::getSingletonPtr()->getByName("GroundPlane")->clone("Grave1").get();
	ground2Mat->getTechnique(0)->getPass(0)->getTextureUnitState(0)->setTextureName("Grave1.png");

	// Grave1 flipped
	ground2Mat = MaterialManager::getSingletonPtr()->getByName("Grave1")->clone("Grave1Flipped").get();
	ground2Mat->getTechnique(0)->getPass(0)->getFragmentProgramParameters()->setNamedConstant("flipped", -1);

	// Grave2
	ground2Mat = MaterialManager::getSingletonPtr()->getByName("GroundPlane")->clone("Grave2").get();
	ground2Mat->getTechnique(0)->getPass(0)->getTextureUnitState(0)->setTextureName("Grave2.png");

	// Grave2 flipped
	ground2Mat = MaterialManager::getSingletonPtr()->getByName("Grave2")->clone("Grave2Flipped").get();
	ground2Mat->getTechnique(0)->getPass(0)->getFragmentProgramParameters()->setNamedConstant("flipped", -1);

	// Grave2
	ground2Mat = MaterialManager::getSingletonPtr()->getByName("GroundPlane")->clone("Grave3").get();
	ground2Mat->getTechnique(0)->getPass(0)->getTextureUnitState(0)->setTextureName("Grave3.png");

	// Grave2 flipped
	ground2Mat = MaterialManager::getSingletonPtr()->getByName("Grave3")->clone("Grave3Flipped").get();
	ground2Mat->getTechnique(0)->getPass(0)->getFragmentProgramParameters()->setNamedConstant("flipped", -1);

	// Grave2
	ground2Mat = MaterialManager::getSingletonPtr()->getByName("GroundPlane")->clone("Grave4").get();
	ground2Mat->getTechnique(0)->getPass(0)->getTextureUnitState(0)->setTextureName("Grave4.png");

	// Grave2 flipped
	ground2Mat = MaterialManager::getSingletonPtr()->getByName("Grave4")->clone("Grave4Flipped").get();
	ground2Mat->getTechnique(0)->getPass(0)->getFragmentProgramParameters()->setNamedConstant("flipped", -1);

	float size = 25;
//	createZombie(size, size*1.58, M_PI_2, Vector3(-50, 60, -50.f));
//	createZombie(size, size*1.58, M_PI_2, Vector3(30, 60, -70.f));
//	createZombie(size, size*1.58, M_PI_2, Vector3(-20, 60, -130.f));
//	createZombie(size, size*1.58, M_PI_2, Vector3(10, 60, -170.f));

	size = 300;
	createGroundPlane(size, size/3.710144928, 0);
	createGroundPlane(size, size/3.710144928, 0);
	createGroundPlane(size, size/3.710144928, 0);
	createGroundPlane(size, size/3.710144928, 0);
	createGroundPlane(size, size/3.710144928, 0);
	createGroundPlane(size, size/3.710144928, 0);
	createGroundPlane(size, size/3.710144928, 0);
	createGroundPlane(size, size/3.710144928, 0);
	createGroundPlane(size, size/3.710144928, 0);

	createRandomEnvItem();
	createRandomEnvItem();
	createRandomEnvItem();
	createRandomEnvItem();
	createRandomEnvItem();
	createRandomEnvItem();
	createRandomEnvItem();
	createRandomEnvItem();
	createRandomEnvItem();
	createRandomEnvItem();
	createRandomEnvItem();
	createRandomEnvItem();
	createRandomEnvItem();
	createRandomEnvItem();
	createRandomEnvItem();
	createRandomEnvItem();
	createRandomEnvItem();
	createRandomEnvItem();
	createRandomEnvItem();
	createRandomEnvItem();
}

SceneNode* GameState::createZombie(float sizeX, float sizeY, float roration, Vector3 pos, String prefix)
{
	m_zombies++;

	String zombieName = prefix+"zombie"+StringConverter::toString(m_zombies);
	MaterialManager::getSingletonPtr()->getByName("BaseZombie")->clone(zombieName);

	Ogre::Plane zombie(Ogre::Vector3::UNIT_Y, 0);
	Ogre::MeshManager::getSingleton().createPlane(zombieName, Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME,
												  zombie, sizeX, sizeY, 1, 1, true, 1, 1, 1, Ogre::Vector3::UNIT_X);
	Ogre::Entity* zombieEntity = m_pSceneMgr->createEntity(zombieName, zombieName);
	Ogre::SceneNode *zombieNode = m_pSceneMgr->getRootSceneNode()->createChildSceneNode(zombieName);
	zombieNode->attachObject(zombieEntity);
	zombieNode->setPosition(pos);
	zombieNode->roll(Radian(M_PI_2));
	zombieNode->pitch(Radian(roration));

	zombieEntity->setMaterialName(zombieName);
	zombieEntity->setCastShadows(false);

	autoNodes.push_back(zombieNode);

	return zombieNode;
}

SceneNode* GameState::createGroundPlane(float sizeX, float sizeY, double posX)
{
	m_worldPlanes++;

	String zombieName = "YWorldPlane"+StringConverter::toString(m_worldPlanes);

	Ogre::Plane zombie(Ogre::Vector3::UNIT_Y, 0);
	Ogre::MeshManager::getSingleton().createPlane(zombieName, Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME,
												  zombie, sizeX, sizeY, 1, 1, true, 1, 1, 1, Ogre::Vector3::UNIT_X);
	Ogre::Entity* zombieEntity = m_pSceneMgr->createEntity(zombieName, zombieName);
	Ogre::SceneNode *zombieNode = m_pSceneMgr->getRootSceneNode()->createChildSceneNode();
	zombieNode->attachObject(zombieEntity);
	zombieNode->setPosition(Vector3(posX, sizeY/2, -m_worldPlanes*60));
	zombieNode->roll(Radian(M_PI_2));
	zombieNode->pitch(Radian(M_PI_2));

	int ground = rand()%4;
	switch (ground)
	{
	case 0:
		zombieEntity->setMaterialName("GroundPlane");
		break;
	case 1:
		zombieEntity->setMaterialName("GroundPlane1");
		break;
	case 2:
		zombieEntity->setMaterialName("GroundPlane2");
		break;
	case 3:
		zombieEntity->setMaterialName("GroundPlane3");
		break;
	}

	zombieEntity->setCastShadows(false);

	autoNodes.push_back(zombieNode);

	return zombieNode;
}

SceneNode* GameState::createRandomEnvItem()
{
	m_worldEnvSpawnLeft = !m_worldEnvSpawnLeft;

	float worldEnvItems = m_worldEnvItems;

	m_worldEnvItems++;

	if(m_worldEnvSpawnLeft)
	{
		worldEnvItems++;
	}

	String itemName = "XEnvItem"+StringConverter::toString(m_worldEnvItems);
	String materialname = "Tree1";
	float sizeX;
	float sizeY;
	float posx;
	float posy;

	sizeX = 14;
	sizeY = 35;
	posx = rand()%60 + 20;
	posy = 60;

	int item = rand()%9;
	switch (item)
	{
	case 0:
		posy = 87;
		sizeX = 170;
		sizeY = 170/1.8;
		if(m_worldEnvSpawnLeft)
		{
			materialname = "Tree1Flipped";
			posx = -65;
		}
		else
		{
			materialname = "Tree1";
			posx = 65;
		}
		break;
	case 1:
		materialname = "Grave1";
		if(m_worldEnvSpawnLeft)
		{
			materialname = "Grave1";
			posx *= -1;
		}
		break;
	case 2:
		materialname = "Grave1Flipped";
		if(m_worldEnvSpawnLeft)
		{
			materialname = "Grave1Flipped";
			posx *= -1;
		}
		break;
	case 3:
		materialname = "Grave2";
		if(m_worldEnvSpawnLeft)
		{
			materialname = "Grave2";
			posx *= -1;
		}
		break;
	case 4:
		materialname = "Grave2Flipped";
		if(m_worldEnvSpawnLeft)
		{
			materialname = "Grave2Flipped";
			posx *= -1;
		}
		break;
	case 5:
		materialname = "Grave3";
		if(m_worldEnvSpawnLeft)
		{
			materialname = "Grave3";
			posx *= -1;
		}
		break;
	case 6:
		materialname = "Grave3Flipped";
		if(m_worldEnvSpawnLeft)
		{
			materialname = "Grave3Flipped";
			posx *= -1;
		}
		break;
	case 7:
		materialname = "Grave4";
		if(m_worldEnvSpawnLeft)
		{
			materialname = "Grave4";
			posx *= -1;
		}
		break;
	case 8:
		materialname = "Grave4Flipped";
		if(m_worldEnvSpawnLeft)
		{
			materialname = "Grave4Flipped";
			posx *= -1;
		}
		break;
	}

	Ogre::Plane zombie(Ogre::Vector3::UNIT_Y, 0);
	Ogre::MeshManager::getSingleton().createPlane(itemName, Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME,
												  zombie, sizeX, sizeY, 1, 1, true, 1, 1, 1, Ogre::Vector3::UNIT_X);
	Ogre::Entity* zombieEntity = m_pSceneMgr->createEntity(itemName, itemName);
	Ogre::SceneNode *zombieNode = m_pSceneMgr->getRootSceneNode()->createChildSceneNode();
	zombieNode->attachObject(zombieEntity);
	zombieNode->setPosition(Vector3(posx, posy, -worldEnvItems*(68/2)));
	zombieNode->roll(Radian(M_PI_2));
	zombieNode->pitch(Radian(M_PI_2));

	zombieEntity->setMaterialName(materialname);

	zombieEntity->setCastShadows(false);

	autoNodes.push_back(zombieNode);

	return zombieNode;
}

void GameState::deleteElementsBehind()
{
	std::vector<SceneNode*>::iterator it = autoNodes.begin();

	while(it < autoNodes.end())
	{
		SceneNode* node = *it;
		if(node)
		{
			bool behind = (m_pCamera->getPosition().z+200) < node->getPosition().z;

			if(behind &&
					(node->getName().at(0) == 'z'||
					 node->getName().at(0) == 'X'||
					 node->getName().at(0) == 'U'||
					 node->getName().at(0) == 'B'||
					 node->getName().at(0) == 'Y')
					)
			{
				try
				{
					m_pSceneMgr->destroySceneNode(node->getName());
				}
				catch(Exception&)
				{
					cerr << "Node " << node->getName() << " already deleted" << endl;
				}

				autoNodes.erase(it);
				return;
			}
		}

		it++;
	}
}


bool GameState::keyPressed(const OIS::KeyEvent &keyEventRef)
{
	if(AdvancedOgreFramework::getSingletonPtr()->m_pKeyboard->isKeyDown(OIS::KC_ESCAPE))
	{
		pushAppState(findByName("PauseState"));
		return true;
	}

	if(m_bSettingsMode && AdvancedOgreFramework::getSingletonPtr()->m_pKeyboard->isKeyDown(OIS::KC_RETURN) ||
			AdvancedOgreFramework::getSingletonPtr()->m_pKeyboard->isKeyDown(OIS::KC_NUMPADENTER))
	{
	}

	if(!m_bSettingsMode || (m_bSettingsMode && !AdvancedOgreFramework::getSingletonPtr()->m_pKeyboard->isKeyDown(OIS::KC_O)))
		AdvancedOgreFramework::getSingletonPtr()->keyPressed(keyEventRef);

	return true;
}

bool GameState::keyReleased(const OIS::KeyEvent &keyEventRef)
{
	AdvancedOgreFramework::getSingletonPtr()->keyPressed(keyEventRef);
	return true;
}

bool GameState::mouseMoved(const OIS::MouseEvent &evt)
{
	if(AdvancedOgreFramework::getSingletonPtr()->m_pTrayMgr->injectMouseMove(evt)) return true;

	if(m_bRMouseDown)
	{
		m_pCamera->yaw(Degree(evt.state.X.rel * -0.1f));
		m_pCamera->pitch(Degree(evt.state.Y.rel * -0.1f));
	}

	return true;
}

bool GameState::mousePressed(const OIS::MouseEvent &evt, OIS::MouseButtonID id)
{
	if(AdvancedOgreFramework::getSingletonPtr()->m_pTrayMgr->injectMouseDown(evt, id)) return true;

	if(id == OIS::MB_Left)
	{
		onLeftPressed(evt);
		// createShere(mSphereNodes.size() + 1, evt);
		m_bLMouseDown = true;
	}
	else if(id == OIS::MB_Right)
	{
		m_bRMouseDown = true;
	}

	return true;
}

bool GameState::mouseReleased(const OIS::MouseEvent &evt, OIS::MouseButtonID id)
{
	if(AdvancedOgreFramework::getSingletonPtr()->m_pTrayMgr->injectMouseUp(evt, id)) return true;

	if(id == OIS::MB_Left)
	{
		m_bLMouseDown = false;
	}
	else if(id == OIS::MB_Right)
	{
		m_bRMouseDown = false;
	}

	return true;
}

void GameState::playZombieAnimations()
{
	std::vector<ZombieAnimationStateIterator> toDelete;

	ZombieAnimationStateIterator it;
	for (it = mPendingAnimations.begin() ; it != mPendingAnimations.end(); ++it)
	{
		long double currentTime = getCurrentTime();
		long double diffTime = currentTime - it->startedAt;
		if(diffTime > it->animationDuration)
		{
			if(it->repeat)
			{
				it->startedAt = currentTime;
			}
			else
			{
				if(it->type == ZOMBIE_ANIMATION_TYPE_BANG || it->type == ZOMBIE_ANIMATION_TYPE_DIE)
				{
					try
					{
						std::vector<SceneNode*>::iterator nodeIt = autoNodes.begin();

						while(nodeIt < autoNodes.end())
						{
							SceneNode* node = *nodeIt;
							if(node && node->getName() == it->name)
							{
								autoNodes.erase(nodeIt);
								break;
							}

							nodeIt++;
						}
						m_pSceneMgr->destroySceneNode(it->name);
					}
					catch(Exception &e)
					{
						cerr << "Node " << it->name << " already deleted" << endl;
					}
				}

				if(it->type == ZOMBIE_ANIMATION_TYPE_SPAWN)
				{
					cout << "HERE" << endl;

					it->type = ZOMBIE_ANIMATION_TYPE_WALK;
					it->repeat = true;
					it->startedAt = currentTime;
					it->numFrames = 9;
					it->startFrame = 15;
					it->endFrame = 22;
					it->animationDuration = 1000;
				}
				else
				{
					toDelete.push_back(it);
				}
			}
			continue;
		}

		int frame = ceil((diffTime/it->animationDuration) * it->numFrames) - 1 + it->startFrame;
		if(it->currentFrame != frame)
		{
			float offsetX = 1.f / it->numFramesX;
			int frameX = frame%it->numFramesX;
			float coordX = offsetX*((float)frameX);

			float offsetY = 1.f / it->numFramesY;
			int frameY = frame/int(it->numFramesX);
			float coordY = offsetY*((float)frameY);

			it->material->getTechnique(0)->getPass(0)->getFragmentProgramParameters()->setNamedConstant("frameX", coordX);
			it->material->getTechnique(0)->getPass(0)->getFragmentProgramParameters()->setNamedConstant("frameY", coordY);
//			cout << "Frame: " << frame+it->startFrame << endl;
//			cout << "Coords: x:" << frameX << " y:" << frameY << endl;

			it->currentFrame = frame;
		}
	}

	std::vector<ZombieAnimationStateIterator>::iterator delIt;
	for (delIt = toDelete.begin() ; delIt != toDelete.end(); ++delIt)
	{
		mPendingAnimations.erase(*delIt);
	}
}

long long GameState::getCurrentTime()
{
	static struct timeval start;

	gettimeofday(&start, NULL);
	return ((start.tv_sec * 1000) + (start.tv_usec / 1000.0));
}

ZombieAnimationState* GameState::pushZombieAnimationState(ZOMBIE_ANIMATION_TYPE type, String name, bool repeat, bool replace)
{
	std::vector<ZombieAnimationState>::iterator it;
	for (it = mPendingAnimations.begin() ; it != mPendingAnimations.end(); ++it)
	{
		if(it->name == name)
		{
			if(it->type == ZOMBIE_ANIMATION_TYPE_DIE || it->type == type)
			{
				return 0;
			}
			if(replace)
			{
				mPendingAnimations.erase(it);
				break;
			}
			return 0;
		}
	}

	ZombieAnimationState animState;
	animState.type = type;
	animState.startedAt = getCurrentTime();
	animState.repeat = repeat;
	animState.name = name;
	animState.numFramesX = 10;
	animState.numFramesY = 4;

	switch (type)
	{
	case ZOMBIE_ANIMATION_TYPE_SPAWN:
		animState.currentFrame = 0;
		animState.numFrames = 14;
		animState.startFrame = 0;
		animState.endFrame = 12;
		animState.animationDuration = 1000;
		break;
	case ZOMBIE_ANIMATION_TYPE_WALK:
		animState.currentFrame = 0;
		animState.numFrames = 9;
		animState.startFrame = 15;
		animState.endFrame = 22;
		animState.animationDuration = 1000;
		break;
	case ZOMBIE_ANIMATION_TYPE_ATTACK:
		animState.currentFrame = 0;
		animState.numFrames = 10;
		animState.startFrame = 23;
		animState.endFrame = 32;
		animState.animationDuration = 1000;
		break;
	case ZOMBIE_ANIMATION_TYPE_DIE:
		animState.currentFrame = 0;
		animState.numFrames = 7;
		animState.startFrame = 33;
		animState.endFrame = 12;
		animState.animationDuration = 500;
		break;
	case ZOMBIE_ANIMATION_TYPE_BANG:
		animState.currentFrame = 0;
		animState.numFrames = 9;
		animState.startFrame = 0;
		animState.endFrame = 8;
		animState.animationDuration = 500;
		animState.numFramesX = 3;
		animState.numFramesY = 3;
		break;
	}

	mPendingAnimations.push_back(animState);

	return &(mPendingAnimations.back());
}

void GameState::onLeftPressed(const OIS::MouseEvent &evt)
{
	float width = m_pCamera->getViewport()->getActualWidth();
	float height = m_pCamera->getViewport()->getActualHeight();
	Ray mouseRay = m_pCamera->getCameraToViewportRay(evt.state.X.abs/float(width), evt.state.Y.abs/float(height));
	m_RayQuery->setRay(mouseRay);

	RaySceneQueryResult &result = m_RayQuery->execute();
	RaySceneQueryResult::iterator itr = result.begin();

	Entity* obj = 0;

	// loop through the results
	for ( itr = result.begin( ); itr != result.end(); itr++)
	{
		if ( itr->movable )
		{
			// TODO better way to distinguish between zombies and rest of entities
			// currently if entity name starts with 'z' it will be treated as zombie
			if(itr->movable->getName().at(0) == 'z')
			{
				obj = m_pSceneMgr->getEntity(itr->movable->getName());

				if(obj)
				{
//					float dist = (obj->getParentNode()->getPosition() - m_pCamera->getPosition()).length();
					{
						Material* mat = obj->getSubEntity(0)->getMaterial().get();

						ZombieAnimationState* animState = pushZombieAnimationState(ZOMBIE_ANIMATION_TYPE_DIE, obj->getName(), false, true);

						if(animState)
						{
							try
							{
								animState->material = mat;
								SceneNode* node = createZombie(100, 100, M_PI_2, Vector3(10, 60, -170.f), "Bang");
								node->roll(Radian(-(M_PI_2)));
								node->yaw(Radian(-M_PI_4));
								node->pitch(Radian(-M_PI_4));
								Entity* entity = m_pSceneMgr->getEntity(node->getName());
								mat = entity->getSubEntity(0)->getMaterial().get();
								node->setPosition(obj->getParentSceneNode()->getPosition() + Vector3(0,0,1));
								animState = pushZombieAnimationState(ZOMBIE_ANIMATION_TYPE_BANG, node->getName(), false, true);
								mat->getTechnique(0)->getPass(0)->getTextureUnitState(0)->setTextureName("Bang_01.png");
								mat->getTechnique(0)->getPass(0)->getFragmentProgramParameters()->setNamedConstant("numFramesX", 3.f);
								mat->getTechnique(0)->getPass(0)->getFragmentProgramParameters()->setNamedConstant("numFramesY", 3.f);

								if(animState)
								{
									animState->material = mat;
								}
							}
							catch(Exception&)
							{

							}
						}
					}

				}
			}
		}
	}



	cout << endl;
}

void GameState::moveCamera()
{
	if(AdvancedOgreFramework::getSingletonPtr()->m_pKeyboard->isKeyDown(OIS::KC_LSHIFT))
		m_pCamera->moveRelative(m_TranslateVector);
	m_pCamera->moveRelative(m_TranslateVector / 10);
}

void GameState::getInput()
{
	if(m_bSettingsMode == false)
	{
		if(AdvancedOgreFramework::getSingletonPtr()->m_pKeyboard->isKeyDown(OIS::KC_A))
			m_TranslateVector.x = -m_MoveScale;

		if(AdvancedOgreFramework::getSingletonPtr()->m_pKeyboard->isKeyDown(OIS::KC_D))
			m_TranslateVector.x = m_MoveScale;

		if(AdvancedOgreFramework::getSingletonPtr()->m_pKeyboard->isKeyDown(OIS::KC_W))
			m_TranslateVector.z = -m_MoveScale;

		if(AdvancedOgreFramework::getSingletonPtr()->m_pKeyboard->isKeyDown(OIS::KC_S))
			m_TranslateVector.z = m_MoveScale;
	}
}

void GameState::controlAnimations()
{
	std::vector<SceneNode*>::iterator it = autoNodes.begin();

	while(it < autoNodes.end())
	{
		SceneNode* node = *it;
		if(node)
		{
			if(node->getName().at(0) == 'z')
			{
				Vector3 objToCamera = (node->getPosition() - m_pCamera->getPosition());

//				cout << objToCamera << endl;
				if(objToCamera.length() < 65)
				{
					Entity* ent = m_pSceneMgr->getEntity(node->getName());
					ZombieAnimationState* animState = pushZombieAnimationState(ZOMBIE_ANIMATION_TYPE_ATTACK, ent->getName(), true, true);
					if(animState)
					{
						Material* mat = ent->getSubEntity(0)->getMaterial().get();

						animState->material = mat;
					}
				}
				else
					if(objToCamera.length() < 300)
				{
					Entity* ent = m_pSceneMgr->getEntity(node->getName());
					ZombieAnimationState* animState = pushZombieAnimationState(ZOMBIE_ANIMATION_TYPE_SPAWN, ent->getName(), false, false);
					if(animState)
					{
						Material* mat = ent->getSubEntity(0)->getMaterial().get();

						animState->material = mat;
					}
				}
			}
		}

		it++;
	}
}

void GameState::update(double timeSinceLastFrame)
{
	deleteElementsBehind();
	int progress = -m_pCamera->getPosition().z / 60;
//	cout << "Progress: " << progress << endl;
//	cout << "Grounds: " << m_worldPlanes << endl;

	while(progress + 30 > m_worldPlanes)
	{
		createGroundPlane(300, 300/3.710144928, 0);
	}

	progress = -m_pCamera->getPosition().z / 68;
	while((progress + 30)*2 > m_worldEnvItems)
	{
		createRandomEnvItem();
	}

	while((progress+15) > m_zombies)
	{
		m_lastShoot = getCurrentTime();
		SceneNode* node = createZombie(25, 25*1.58, M_PI_2, Vector3(rand()%40 - 20, 60, m_pCamera->getPosition().z - rand()%500 - 1000));
	}

	controlAnimations();

	m_FrameEvent.timeSinceLastFrame = timeSinceLastFrame;
	AdvancedOgreFramework::getSingletonPtr()->m_pTrayMgr->frameRenderingQueued(m_FrameEvent);

	if(m_bQuit == true)
	{
		popAppState();
		return;
	}

	m_MoveScale = m_MoveSpeed * timeSinceLastFrame;
	m_RotScale = m_RotateSpeed * timeSinceLastFrame;
	m_MoveUpScale = sin(m_pCamera->getPosition().z - m_pCamera->getPosition().z / m_MoveSpeed / 1.1);

	m_TranslateVector = Vector3::ZERO;

	Vector3 camPos= m_pCamera->getPosition();
	camPos.y = 65 + m_MoveUpScale;
	m_pCamera->setPosition(camPos);

	getInput();
	moveCamera();
	playZombieAnimations();
}

bool GameState::frameStarted(const FrameEvent &evt)
{

//	phyWorld->stepSimulation(evt.timeSinceLastFrame*2, 10);
//	phyWorld->debugDrawWorld();

	//Shows debug if F3 key down.
//	dbgdraw->setDebugMode(AdvancedOgreFramework::getSingletonPtr()->m_pKeyboard->isKeyDown(OIS::KC_F3));
//	dbgdraw->step();

	return true;
}

void GameState::buildGUI()
{
//	AdvancedOgreFramework::getSingletonPtr()->m_pTrayMgr->showFrameStats(OgreBites::TL_TOPLEFT);
	AdvancedOgreFramework::getSingletonPtr()->m_pTrayMgr->showCursor();
//	AdvancedOgreFramework::getSingletonPtr()->m_pTrayMgr->createButton(OgreBites::TL_TOPRIGHT, "ResetBtn", "Reset Car");
}
void GameState::buttonHit(OgreBites::Button *button)
{
//	if(button->getName() == "ResetBtn")
//	{
//		mVehicleController->clientResetScene();
//	}
}

void GameState::sliderMoved(OgreBites::Slider *slider)
{
//	mVehicleController->wheelFriction = wheelFrictionSlider->getValue();
//	mVehicleController->wheelRadius = wheelRadiusSlider->getValue();
//	mVehicleController->suspensionStiffness = suspensionStiffnessSlider->getValue();
//	mVehicleController->suspensionCompression = suspensionCompressionSlider->getValue();
//	mVehicleController->suspensionDamping = suspensionDampingSlider->getValue();
//	mVehicleController->rollInfluence = rollInfluenceSlider->getValue();
	
//	mVehicleController->updateWheelsParams();
}
